/// @addtogroup dynsys
/// @{

/////////////////////////////////////////////////////////////////////////////
/// @file cndynsys.h
///
/// @author Daniel Wilczak
/////////////////////////////////////////////////////////////////////////////

// Copyright (C) 2000-2005 by the CAPD Group.
//
// This file is part of the CAPD library.  This library is free software;
// you can redistribute it and/or modify it under the terms of the GNU
// General Public License as published by the Free Software Foundation;
// either version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this software; see the file "license.txt".  If not, write to the
// Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
// MA 02111-1307, USA.

/** Author: Daniel Wilczak, 2006 */

#ifndef _CnDynSys_H_
#define _CnDynSys_H_

#include "dynsys/c2dynsys.h"
#include "map/CnCoeff.h"

namespace capd{
namespace dynsys{

template<typename MatrixT>
class CnDynSys : public virtual capd::dynsys::C2DynSys<MatrixT>{
public:
   typedef MatrixT MatrixType;
   typedef typename MatrixType::RowVectorType VectorType;
   typedef typename MatrixType::ScalarType ScalarType;
   typedef Norm<VectorType,MatrixType> NormType;
   typedef capd::map::C2Coeff<ScalarType> C2CoeffType;
   typedef capd::map::CnCoeff<MatrixType> CnCoeffType;

   virtual void cnPhi(const VectorType&, CnCoeffType&) const = 0;
   virtual void cnEnclosure(const VectorType& x, CnCoeffType& result, NormType&) const = 0;
   virtual void cnRemainder(const VectorType& x, NormType&, CnCoeffType& result) const = 0;

//from C2DynSys
   using C2DynSys<MatrixType>::c2Phi;
   using C2DynSys<MatrixType>::c2Enclosure;
   using C2DynSys<MatrixType>::c2Remainder;

//from C1DynSys
   using C1DynSys<MatrixType>::JacRemainder;
   using C1DynSys<MatrixType>::jacEnclosure;
   using C1DynSys<MatrixType>::enclosure;

// from DynSys
   using C1DynSys<MatrixType>::Phi;
   using C1DynSys<MatrixType>::JacPhi;
   using C1DynSys<MatrixType>::Lipschitz;
   using C1DynSys<MatrixType>::type;
};

}} // namespace capd::dynsys

#endif

/// @}
