/// @addtogroup dynsys
/// @{

/////////////////////////////////////////////////////////////////////////////
/// @file taylorex.h
///
/// @author Daniel Wilczak
/////////////////////////////////////////////////////////////////////////////

// Copyright (C) 2000-2005 by the CAPD Group.
//
// This file is part of the CAPD library.  This library is free software;
// you can redistribute it and/or modify it under the terms of the GNU
// General Public License as published by the Free Software Foundation;
// either version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this software; see the file "license.txt".  If not, write to the
// Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
// MA 02111-1307, USA.

#ifndef _TaylorException_H
#define _TaylorException_H

#include <string>
#include <sstream>
#include <fstream>
#include <stdexcept>

namespace capd{
namespace dynsys{

template<typename VectorType>
class TaylorException : public std::runtime_error{
public:
   typedef typename VectorType::ScalarType ScalarType;

   VectorType V;
   ScalarType step;
   std::string message;

   TaylorException(const std::string &info, const VectorType &_V, const ScalarType &S)
      : std::runtime_error(info), V(_V), step(S)
   {
      std::ostringstream d;
      d << std::runtime_error::what() << "\n";
      d << "the set: " << V << "\n";
      d << "time step: " << step << "\n";
      message = d.str();
   }

   ~TaylorException() throw() {}

   const char* what() const throw()
   {
      return message.c_str();
   }
};

}} // namespace capd::dynsys

#endif

/// @}
