#include "typedefs.h"
#include "utils.hpp"
#include "algorithms.hpp"
#include "computeGMap.hpp"

#include "nonrigorous.c"
#include "init.c"
#include "computeDerivatives.c"
#include "mainProcedures.c"
#include "algorithm1.c"
#include "algorithm2.c"

int main(int,char**)
{  
   std::cout.precision(20);
   try{
      std::string rosslerFormula="var:x,y,z,a,b;fun:-(y+z),x+b*y,b+z*(x-a);";
      std::string rosslerFormula4D="var:x,y,z,a,b;fun:-y-z,x+b*y,b+z*(x-a),0;";
      std::string section = "var:x,y,z;fun:x;";
      std::string section4D = "var:x,y,z,a;fun:x;";

      BasicPoincareMap BPM(rosslerFormula,section,10,0.1);
      PoincareMap PM(rosslerFormula,section,6,0.1,0.1);
      BPM.setParameter("b",0.2);
      PM.setParameter("b",interval(2)/interval(10));

      PoincareMap PM4D(rosslerFormula4D,section4D,6,0.1);
      PM4D.setParameter("b",interval(2)/interval(10));

      double firstBif_mina = 2.83244;
      double firstBif_maxa = 2.832446;
      double secondBif_mina = 3.83735812;
      double secondBif_maxa = 3.837358168411;
      MainData md1, md2;

      initFirstBif(BPM,md1,firstBif_mina,firstBif_maxa);
      initSecondBif(BPM,md2,secondBif_mina,secondBif_maxa);
      double T = time(0);

// firstBifurcation
      checkMinMaxA(BPM,PM,md1);
      checkDGX(PM,md1);
      checkMonotonicity(PM4D,PM,md1);
      computeEigenvalues(PM,PM4D,md1);
      checkA4aCondition(PM,md1);

// secondBifurcation
      checkMinMaxA(BPM,PM,md2);
      checkDGX(PM,md2);
      checkMonotonicity(PM4D,PM,md2);
      computeEigenvalues(PM,PM4D,md2);
      checkA4aCondition(PM,md2);

// connecting branch of period two points

// continuation of connecting branch using C^3 computations
      proveConnectingBranch(BPM,PM,PM4D,md1);
// the existence of connecting branch far away from the first bifurcation point, Krawczyk method
      connectingBranch(BPM,PM,md1,md2);

      T = time(0)-T;
      std::cout << "\n Time of computations: " << T << "seconds = " << T/60 << " minutes" << std::endl;
   }catch(std::exception& e)
   {
      std::cout << e.what() << std::endl;
   }
   return 0;
}
