// this file contains two procedueres which initialize data structure MainData& md
// this stucture contains all informations about the coorinate system and the set
// on which we verify the existence of period doubling bifurcation
// the arguments are
// BPM - an object for computing nonrigorously Poincare map
// mina and maxa - range of parameters in which we verify the existence of PDB

void initFirstBif(BasicPoincareMap& BPM, MainData& md, double mina, double maxa)
{
   md.bifNumber = 1;
   md.mina = mina;
   md.maxa = maxa;

// good candidate for initial condition in nonrigorous Newton method which finds approximate fixed point
   md.startPoint= DVector(0.,-4.777,0.06);
   md.minaStep  = 0.1;   
   md.minaOrder = 10;

   md.maxaStep  = 0.05;  
   md.maxaOrder = 14;

   md.d3gStep   = 0.04;  
   md.d3gOrder  = 6; 
   md.d3gGridX  = 30; 
   md.d3gGridA  = 1;

   md.dxa3DStep = 0.02;
   md.dxa4DStep = 0.01;
   md.dxa3DOrder= 14;
   md.dxa4DOrder= 10;
   md.dxaGridA  = 1;

   md.dgxStep   = 0.05;
   md.dgxOrder  = 10;

   md.d2g3DStep = 0.05;
   md.d2g4DStep = 0.05;
   md.d2g3DOrder= 6;
   md.d2g4DOrder= 6;
   md.d2gGridA  = 6;
   md.d2gGridX  = 30;

   md.ev3DStep  = 0.03;
   md.ev4DStep  = 0.03;
   md.ev3DOrder = 10;
   md.ev4DOrder = 10;
   md.evGridA   = 1;

   md.a4aStep   = 0.03;
   md.a4aOrder  = 10;
   md.a4aGridX  = 3;
   md.a4aGridA  = 1;

   md.coordinateSystem  = "firstBifCoordSystem.dat";
   md.eigenvalues       = "firstBifEigenvalues.dat";
   md.d2gd3gEstimations = "firstBif_D2G_D3G.dat";
   md.d2gdxaEstimation  = "firstBif_D2G_DXA.dat";
   md.dgdxOnA1          = "firstBif_DGDX_OnA1.dat";
   md.a4aEstimation     = "firstBifA4aEstimation.dat";

   // we choose a point from which we start searching for a fixed point

   BPM.setParameter("a",2.832445);
   md.startPoint = findPeriodicOrbit(BPM,md.startPoint,2*md.bifNumber);
   md.Ix[0] = md.Dx[0] = md.startPoint[1];
   md.Ix[1] = md.Dx[1] = md.startPoint[2];

// compute derivative at the approximate fixed point
// eigenvectors will give us coordinate system
   DMatrix DP(3,3);
   computePoincare(BPM,2,md.startPoint,DP);

   DMatrix CS = take2DMatrix(DP);
   md.DCoordSystem = eigenvectors(CS);

   md.ICoordSystem(1,1) = md.DCoordSystem(1,1);
   md.ICoordSystem(1,2) = md.DCoordSystem(1,2);
   md.ICoordSystem(2,1) = md.DCoordSystem(2,1);
   md.ICoordSystem(2,2) = md.DCoordSystem(2,2);
   
   md.IInvCS = inverse2x2Matrix(md.ICoordSystem);
   md.DInvCS = inverse2x2Matrix(md.DCoordSystem);

// definition of the set around fixed point
   double size=1.3107e-3;
   md.box[0] = interval(-1.,1.)*size;
   md.box[1] = interval(-1.,1.)*size/10;

// we create a grid dgxGridX

   int s = 3;
   interval stepX = diam(md.box[0])/s;
   interval boxX = stepX*interval(0,1);

   for(int p=0;p<s;++p)
   {
      interval x = md.box[0].leftBound() + p*stepX + boxX;
      int z=8000/(1+(p-s/2)*(p-s/2));
      interval step = diam(x)/z;
      interval b = interval(0,1)*step;
      for(int q=0;q<z;++q)
      {
         md.dgxGridX.push_back(x.leftBound()+q*step+b);
      }
   }
   std::ofstream aFile;
   aFile.open(md.coordinateSystem.c_str());
   aFile.precision(20);
   aFile << "center: " << md.Dx << std::endl;
   aFile << "coordSystem: " << md.DCoordSystem << std::endl;
   aFile.close();
}

// ----------------------------------------------------------------------------------
// similar procedure for second period doubling
void initSecondBif(BasicPoincareMap& BPM, MainData& md, double mina, double maxa)
{
   md.bifNumber = 2;
   md.mina = mina;
   md.maxa = maxa;
   md.startPoint = DVector(0.,-4.777,0.06);

// parameter settings
   md.minaStep   = 0.08;
   md.minaOrder  = 25;

   md.maxaStep   = 0.05;
   md.maxaOrder  = 25;

   md.d3gStep    = 0.04;
   md.d3gOrder   = 10;
   md.d3gGridX   = 1;
   md.d3gGridA   = 150;

   md.dxa3DStep  = 0.02;
   md.dxa4DStep  = 0.01;
   md.dxa3DOrder = 20;
   md.dxa4DOrder = 20;
   md.dxaGridA   = 4;

   md.dgxStep    = 0.05;
   md.dgxOrder   = 14;

   md.d2g3DStep  = 0.05;
   md.d2g4DStep  = 0.05;
   md.d2g3DOrder = 10;
   md.d2g4DOrder = 10;
   md.d2gGridA   = 150;
   md.d2gGridX   = 1;

   md.ev3DStep   = 0.05;
   md.ev4DStep   = 0.01;
   md.ev3DOrder  = 20;
   md.ev4DOrder  = 10;
   md.evGridA    = 4;

   md.a4aStep    = 0.03;
   md.a4aOrder   = 10;
   md.a4aGridX   = 3;
   md.a4aGridA   = 10;

// files
   md.coordinateSystem  = "secondBifCoordSystem.dat";
   md.eigenvalues       = "secondBifEigenvalues.dat";
   md.d2gd3gEstimations = "secondBif_D2G_D3G.dat";
   md.d2gdxaEstimation  = "secondBif_D2G_DXA.dat";
   md.dgdxOnA1          = "secondBif_DGDX_OnA1.dat";
   md.a4aEstimation     = "secondBifA4aEstimation.dat";


   BPM.setParameter("a",maxa);
   md.startPoint = findPeriodicOrbit(BPM,md.startPoint,2*md.bifNumber);
   md.Ix[0] = md.Dx[0] = md.startPoint[1];
   md.Ix[1] = md.Dx[1] = md.startPoint[2];

   DMatrix DP(3,3);
   computePoincare(BPM,2*md.bifNumber,md.startPoint,DP);

   DMatrix CS = take2DMatrix(DP);
   md.DCoordSystem = eigenvectors(CS);

   md.ICoordSystem(1,1) = md.DCoordSystem(1,1);
   md.ICoordSystem(1,2) = md.DCoordSystem(1,2);
   md.ICoordSystem(2,1) = md.DCoordSystem(2,1);
   md.ICoordSystem(2,2) = md.DCoordSystem(2,2);

   md.IInvCS = inverse2x2Matrix(md.ICoordSystem);
   md.DInvCS = inverse2x2Matrix(md.DCoordSystem);

   double size=1.1e-6;
   md.box[0] = interval(-1.,1.)*size;
   md.box[1] = interval(-1.,1.)*size/3;

// we create a grid dgxGridX

   int s = 1;
   interval stepX = diam(md.box[0])/s;
   interval boxX = stepX*interval(0,1);

   for(int p=0;p<s;++p)
   {
      interval x = md.box[0].leftBound() + p*stepX + boxX;
      int z=15000/(1+(p-s/2)*(p-s/2));
      interval step = diam(x)/z;
      interval b = interval(0,1)*step;
      for(int q=0;q<z;++q)
      {
         md.dgxGridX.push_back(x.leftBound()+q*step+b);
      }
   }

   std::ofstream aFile;
   aFile.open(md.coordinateSystem.c_str());
   aFile.precision(20);
   aFile << "center: " << md.Dx << std::endl;
   aFile << "coordSystem: " << md.DCoordSystem << std::endl;
   aFile.close();

}
