// this file defines some nonrigorous routines use in the computer assisted proof

// ------------------------------------------------------------------------
// this is a nonrigorous routine
// it computes suitable iteration of the Poincare Map

void computePoincare(BasicPoincareMap& BPM, int iter, DVector& v)
{
   for(int i=0;i<iter;++i)
      v = BPM(v);
}

// ----------------------------------------------------------------------------

// this is a nonrigorous routine
// it computes suitable iteration of the Poincare Map
// and its derivative

void computePoincare(BasicPoincareMap& BPM, int iter, DVector& v, DMatrix& m)
{
   DCnCoeff c(3,1);
   c()=v;
   c.setMatrix(DMatrix::Identity(3));
   for(int i=0;i<iter;++i)
   {
      c = BPM(c);
   }
   c = BPM.computePartialDerivatives(c);
   v = c();
   m = DMatrix(c);
}

// ----------------------------------------------------------------------------

// this is a nonrigorous routine
// it finds an approximate periodic orbit using standard Newton method

DVector findPeriodicOrbit(BasicPoincareMap& BPM, DVector b,int iter, int steps=15)
{
   DMatrix m(3,3);
   Vector<double,0> Nx(2), Fx(2), x(2);

   for(int i=0;i<steps;++i)
   {
      b[0] = 0; // b[0]=0 is a section

      x = take2DVector(b);
      computePoincare(BPM,iter,b,m);

      // F = P^2 - Id
      Fx = take2DVector(b)-x;

      // DF  = DP^2 - I
      DMatrix DF = take2DMatrix(m)-DMatrix::Identity(2);
      DMatrix invDF = inverse2x2Matrix(DF);

      // newton operator Nx = x - (DF(x))^{-1}*F(x)
      Nx = x - invDF*Fx;
      b[1]=Nx[0];
      b[2]=Nx[1];
   }
   b[0]=0;
   return b;
}

// ----------------------------------------------------------------------------

// this function writes an interval to a given stream (usually file)
template<typename IntervalType>
void writeInterval(std::ostream& out, const IntervalType& intv)
{
   int i=0,j=0;
   typename IntervalType::BoundType l = intv.leftBound();
   typename IntervalType::BoundType r = intv.rightBound();

   while(capd::abs(l)<1 || capd::abs(r)<1)
   {
      ++i;
      ++j;
      l*=10;
      r*=10;
   }
   int p = capd::max(i,j);
   out << "["<< l << "," << r <<"]";
   if(p>0)
      out << "\\cdot 10^{-" << p << "}";
}

