#include <iostream>
#include <fstream>
#include <cstdlib>
#include <ctime>
#include <string>
#include <vector>
#include <gmpxx.h>
#include <mpfr.h>
#include <omp.h>

#ifndef _CAPD_TYPEDEFS_H
#define _CAPD_TYPEDEFS_H

#include "interval/mpInterval.h"
#include "interval/doubleInterval.h"
typedef capd::intervals::Interval<capd::multiPrec::MpfrClass> MPInterval;

#include "vectalg/vector.h"
#include "vectalg/matrix.h"
#include "vectalg/norm.h"
#include "map/CnContainer.h"
#include "map/CnCoeff.h"
#include "map/CnMap.h"
#include "map/function.h"
#include "dynsys/basicTaylor.h"
#include "dynsys/taylor.h"
#include "dynsys/cntaylor.h"
#include "dynset/cnrect2.h"
#include "dynset/c1rect2.h"
#include "dynset/rect2set.h"
#include "poincare/poincare.h"
#include "normalForms/linearSubstitution.hpp"

const int DIMENSION=0;

typedef Matrix<double,DIMENSION,DIMENSION> DMatrix;
typedef Vector<double,DIMENSION> DVector;
typedef Matrix<interval,DIMENSION,DIMENSION> IMatrix;
typedef Vector<interval,DIMENSION> IVector;
typedef Matrix<MPInterval,DIMENSION,DIMENSION> MPMatrix;
typedef Vector<MPInterval,DIMENSION> MPVector;

typedef std::vector<IVector> VectorContainer;
typedef MaxLNorm<IVector,IMatrix> MLNorm;
typedef EuclLNorm<IVector,IMatrix> ELNorm;

typedef capd::map::CnContainer<interval> CnContainer;
typedef capd::map::CnCoeff<IMatrix> CnCoeff;
typedef capd::map::CnCoeff<DMatrix> DCnCoeff;
typedef capd::map::CnMap<IMatrix,3> CnMap;

typedef capd::dynsys::CnTaylor<capd::map::CnMap<Matrix<MPInterval,DIMENSION,DIMENSION> > > MPCnTaylor;
typedef capd::dynsys::BasicCnTaylor<capd::map::CnMap<Matrix<double,DIMENSION,DIMENSION> > > BasicCnTaylor;
typedef capd::dynsys::CnTaylor<CnMap> CnTaylor;

typedef capd::dynset::CnRect2<IMatrix> CnRect2;
typedef capd::dynset::C1rect2<IMatrix> C1Rect2;
typedef capd::dynset::Rect2Set<IMatrix> Rect2Set;

typedef capd::poincare::BasicPoincareMap<BasicCnTaylor> BasicPoincareMap;
typedef capd::poincare::PoincareMap<CnTaylor> PoincareMap;
typedef capd::poincare::PoincareMap<MPCnTaylor> MPPoincareMap;

struct MainData
{
   int bifNumber;
   DVector startPoint; // for searching fixed point

   double mina, maxa;
   IVector Ix, box; // center
   DVector Dx;
   IMatrix ICoordSystem;
   DMatrix DCoordSystem;

   IMatrix IInvCS;
   DMatrix DInvCS;

// parameters for the routines for verifying our assumptions

// PoincaerMap settings for the computations at mina and maxa
   double minaStep;
   int minaOrder;
   double maxaStep;
   int maxaOrder;

// PoincareMap settings for D3G computations
   double d3gStep;
   int d3gOrder;
   int d3gGridX;
   int d3gGridA;

// PoincareMap settings for DXA computations
   double dxa3DStep;
   double dxa4DStep;
   int dxa3DOrder;
   int dxa4DOrder;
   int dxaGridA;

// PoincareMap settings for DGX computations
   double dgxStep;
   int dgxOrder;
   //int dgxGridX;
   std::vector<interval> dgxGridX;

// PoincareMap settings for D2GDXDA computations
   double d2g3DStep;
   double d2g4DStep;
   int d2g3DOrder;
   int d2g4DOrder;
   int d2gGridA;
   int d2gGridX;

// PoincareMap settings for eigenvalues computations
   double ev3DStep;
   double ev4DStep;
   int ev3DOrder;
   int ev4DOrder;
   int evGridA;

// PoincareMap settings for A4a condition
   double a4aStep;
   int a4aOrder;
   int a4aGridX;
   int a4aGridA;
   
// fileName
   std::string coordinateSystem;
   std::string eigenvalues;
   std::string d2gd3gEstimations;
   std::string d2gdxaEstimation;
   std::string dgdxOnA1;
   std::string a4aEstimation;

   MainData() 
    : startPoint(3),
      Ix(2), box(2), Dx(2), 
      ICoordSystem(2,2), DCoordSystem(2,2), 
      IInvCS(2,2), DInvCS(2,2)
   {}
};

inline MPInterval convertInterval(const interval& r)
{
   return MPInterval(r.leftBound(),r.rightBound());
}

struct MPMainData : public MainData
{
   MPVector Ix, box;
   MPMatrix ICoordSystem, IInvCS;
   MPMainData(const MainData& mp) : MainData(mp), Ix(2), box(2), ICoordSystem(2,2), IInvCS(2,2)
   {
      for(int i=0;i<2;++i)
      {
         Ix[i] = convertInterval(MainData::Ix[i]);
         box[i] = convertInterval(MainData::box[i]);
         for(int j=0;j<2;++j)
         {
            ICoordSystem[i][j] = convertInterval(MainData::ICoordSystem[i][j]);
            IInvCS[i][j] = convertInterval(MainData::IInvCS[i][j]);
         }
      }
   }
};

#endif // define _CAPD_TYPEDEFS_H
